# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: kaskada/kaskada/v2alpha/shared.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.api import field_behavior_pb2 as google_dot_api_dot_field__behavior__pb2
from validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n$kaskada/kaskada/v2alpha/shared.proto\x12\x17kaskada.kaskada.v2alpha\x1a\x1fgoogle/api/field_behavior.proto\x1a\x17validate/validate.proto\"4\n\x0eRequestDetails\x12\"\n\nrequest_id\x18\x01 \x01(\tB\x03\xe0\x41\x03R\trequestId\"\xbd\x02\n\x0cSliceRequest\x12N\n\x07percent\x18\x01 \x01(\x0b\x32\x32.kaskada.kaskada.v2alpha.SliceRequest.PercentSliceH\x00R\x07percent\x12X\n\x0b\x65ntity_keys\x18\x02 \x01(\x0b\x32\x35.kaskada.kaskada.v2alpha.SliceRequest.EntityKeysSliceH\x00R\nentityKeys\x1a\x41\n\x0cPercentSlice\x12\x31\n\x07percent\x18\x01 \x01(\x01\x42\x17\xfa\x42\x14\x12\x12\x19\x00\x00\x00\x00\x00\x00Y@!\x00\x00\x00\x00\x00\x00\x00\x00R\x07percent\x1a\x32\n\x0f\x45ntityKeysSlice\x12\x1f\n\x0b\x65ntity_keys\x18\x02 \x03(\tR\nentityKeysB\x0c\n\x05slice\x12\x03\xf8\x42\x01\x42\xf7\x01\n\x1b\x63om.kaskada.kaskada.v2alphaB\x0bSharedProtoP\x01ZMgithub.com/kaskada/kaskada-ai/wren/gen/kaskada/kaskada/v2alpha;kaskadav2alpha\xa2\x02\x03KKX\xaa\x02\x17Kaskada.Kaskada.V2alpha\xca\x02\x17Kaskada\\Kaskada\\V2alpha\xe2\x02#Kaskada\\Kaskada\\V2alpha\\GPBMetadata\xea\x02\x19Kaskada::Kaskada::V2alphab\x06proto3')



_REQUESTDETAILS = DESCRIPTOR.message_types_by_name['RequestDetails']
_SLICEREQUEST = DESCRIPTOR.message_types_by_name['SliceRequest']
_SLICEREQUEST_PERCENTSLICE = _SLICEREQUEST.nested_types_by_name['PercentSlice']
_SLICEREQUEST_ENTITYKEYSSLICE = _SLICEREQUEST.nested_types_by_name['EntityKeysSlice']
RequestDetails = _reflection.GeneratedProtocolMessageType('RequestDetails', (_message.Message,), {
  'DESCRIPTOR' : _REQUESTDETAILS,
  '__module__' : 'kaskada.kaskada.v2alpha.shared_pb2'
  # @@protoc_insertion_point(class_scope:kaskada.kaskada.v2alpha.RequestDetails)
  })
_sym_db.RegisterMessage(RequestDetails)

SliceRequest = _reflection.GeneratedProtocolMessageType('SliceRequest', (_message.Message,), {

  'PercentSlice' : _reflection.GeneratedProtocolMessageType('PercentSlice', (_message.Message,), {
    'DESCRIPTOR' : _SLICEREQUEST_PERCENTSLICE,
    '__module__' : 'kaskada.kaskada.v2alpha.shared_pb2'
    # @@protoc_insertion_point(class_scope:kaskada.kaskada.v2alpha.SliceRequest.PercentSlice)
    })
  ,

  'EntityKeysSlice' : _reflection.GeneratedProtocolMessageType('EntityKeysSlice', (_message.Message,), {
    'DESCRIPTOR' : _SLICEREQUEST_ENTITYKEYSSLICE,
    '__module__' : 'kaskada.kaskada.v2alpha.shared_pb2'
    # @@protoc_insertion_point(class_scope:kaskada.kaskada.v2alpha.SliceRequest.EntityKeysSlice)
    })
  ,
  'DESCRIPTOR' : _SLICEREQUEST,
  '__module__' : 'kaskada.kaskada.v2alpha.shared_pb2'
  # @@protoc_insertion_point(class_scope:kaskada.kaskada.v2alpha.SliceRequest)
  })
_sym_db.RegisterMessage(SliceRequest)
_sym_db.RegisterMessage(SliceRequest.PercentSlice)
_sym_db.RegisterMessage(SliceRequest.EntityKeysSlice)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\033com.kaskada.kaskada.v2alphaB\013SharedProtoP\001ZMgithub.com/kaskada/kaskada-ai/wren/gen/kaskada/kaskada/v2alpha;kaskadav2alpha\242\002\003KKX\252\002\027Kaskada.Kaskada.V2alpha\312\002\027Kaskada\\Kaskada\\V2alpha\342\002#Kaskada\\Kaskada\\V2alpha\\GPBMetadata\352\002\031Kaskada::Kaskada::V2alpha'
  _REQUESTDETAILS.fields_by_name['request_id']._options = None
  _REQUESTDETAILS.fields_by_name['request_id']._serialized_options = b'\340A\003'
  _SLICEREQUEST_PERCENTSLICE.fields_by_name['percent']._options = None
  _SLICEREQUEST_PERCENTSLICE.fields_by_name['percent']._serialized_options = b'\372B\024\022\022\031\000\000\000\000\000\000Y@!\000\000\000\000\000\000\000\000'
  _SLICEREQUEST.oneofs_by_name['slice']._options = None
  _SLICEREQUEST.oneofs_by_name['slice']._serialized_options = b'\370B\001'
  _REQUESTDETAILS._serialized_start=123
  _REQUESTDETAILS._serialized_end=175
  _SLICEREQUEST._serialized_start=178
  _SLICEREQUEST._serialized_end=495
  _SLICEREQUEST_PERCENTSLICE._serialized_start=364
  _SLICEREQUEST_PERCENTSLICE._serialized_end=429
  _SLICEREQUEST_ENTITYKEYSSLICE._serialized_start=431
  _SLICEREQUEST_ENTITYKEYSSLICE._serialized_end=481
# @@protoc_insertion_point(module_scope)
